/*
    Copyright (C) 2006 Manuel Lausch

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Manuel Lausch
    Rudolfstr. 9
    75177 Pforzheim
    GERMANY
    software@manuellausch.de
*/

#include <iostream>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <SDL_thread.h>
#include <SDL_ttf.h>
#include "snake.h"

using std::cerr;
using std::endl;
using std::cout;

SDL_Surface * initMyGame()
{
	SDL_Surface * pDisplay;

	if(SDL_Init(SDL_INIT_VIDEO) < 0)
	{
		cerr << "SDL Konnte nicht initialisiert werden:  " << SDL_GetError() << endl;
		exit(-1);
	}

	if(TTF_Init() < 0)
	{
		cerr << "TTF konnte nicht initialisiert werden:  " << TTF_GetError() << endl;
		exit(-1);
	}

	atexit(SDL_Quit);
	atexit(TTF_Quit);

	if (!(pDisplay = SDL_SetVideoMode(QUADSIZE*PLAYGROUND_WIDTH, QUADSIZE*PLAYGROUND_HEIGHT + 50, 16, SDL_SWSURFACE)))
	{
		cerr << "Video Mode konnte nicht initialisiert werden:  " << SDL_GetError() << endl;
		exit(-1);
	}

	hlineColor(pDisplay, 0, QUADSIZE*PLAYGROUND_WIDTH, QUADSIZE*PLAYGROUND_HEIGHT, 0xffffffff);
	SDL_Flip(pDisplay);

	SDL_WM_SetCaption("Snake 0.3.2", NULL);
	SDL_WM_SetIcon(SDL_LoadBMP("./icon.bmp"), NULL);
	SDL_ShowCursor(SDL_DISABLE);
	return pDisplay;
}

void help();

int main(int argc, char** argv)
{
	const int MAXPLAYER = 2; // max amount of players, it depends on the defined colors and keys
	int playercolor[] = {0x0000ffff , 0xffff00ff };
	const int speed = 50;
	int ende, taste, num_player = 1, LevelNumberVonArgv = 0;

	for(int i = 1; i < argc; i++)
	{
		if(argv[i][0] == '-')
		{
			switch(argv[i][1])
			{
				case 'P':
					num_player = atoi(argv[i + 1]);
					break;

				case 'L':
					LevelNumberVonArgv = i + 1;
					break;

				case 'h':
				case '?':
					help();
					return 0;
					break;
			}
		}
	}

	if(MAXPLAYER < num_player)
	{
		cout << "Es koennen maximal " << MAXPLAYER << " Spieler mitspielen." << endl;
		return 0;
	}
	if(num_player == 0)
	{
		cout << "Wie bitte, soll man ohne Spieler spielen???" << endl;
		return 0;
	}
	if(num_player < 0)
	{
		cout << "Wenn Sie mir veraten, wie ein negativer Spieler aussieht, kann ich den gerne einbauen..." << endl;
		return 0;
	}

	SDL_Surface *pDisplay;
	SDL_Event event;
	snake *player[num_player];
	SDL_Thread *thread[num_player];
	int pause = 0;
	SDL_mutex *mutex;

	// init playground
	pDisplay = initMyGame();
	playground *pPlayground;
	if(LevelNumberVonArgv != 0)
		pPlayground = new playground(pDisplay, argv[LevelNumberVonArgv]);
	else
		pPlayground = new playground(pDisplay);

	mutex = SDL_CreateMutex();

	// create players
	for(int i = 0; i < num_player; i++)
	{
		player[i] = new snake(pDisplay, pPlayground, mutex);
		player[i]->set_speed(speed);
		player[i]->set_color(playercolor[i]);
		thread[i] = SDL_CreateThread(snake::start, player[i]);
	}

	ende = 0;
	SDL_EventState(SDL_KEYUP, SDL_IGNORE); // Wir wollen nur Tastendrcke verarbeiten.
	do
	{
		 SDL_WaitEvent(&event);
		 switch(event.type)
		{
			case SDL_KEYDOWN:
				taste = event.key.keysym.sym;
				break;
			case SDL_QUIT:
				ende += 1;
				break;
			default:
				taste = 0;
		}

		// Pause Funktion
		// wird durch blockieren des Mutex geregelt.

		// ! Hier sollten dann aber auch alle weiter bettigten Tasten ignoeriert werden.
		if (taste == ' ')
		{
			switch(pause)
			{
			case 0:
				pause = 1;
				SDL_mutexP(mutex);
				break;
			case 1:
				pause = 0;
				SDL_mutexV(mutex);
				break;
			}
		}
		// ! Hier sollten dann aber auch alle weiter bettigten Tasten ignoeriert werden.
		if (pause == 1) continue; 	// was hiermit quick & dirty durch abbruch weiterer
								// verarbeitung im pause mode geschehen ist :)

		switch(taste)
		{
			case 273: player[0]->ways.push(up); break;
			case 274: player[0]->ways.push(down); break;
			case 275: player[0]->ways.push(right); break;
			case 276: player[0]->ways.push(left); break;
		}

		if(num_player > 1)
		{
			switch(taste)
			{
				case 'w': player[1]->ways.push(up); break;
				case 's': player[1]->ways.push(down); break;
				case 'd': player[1]->ways.push(right); break;
				case 'a': player[1]->ways.push(left); break;

			}
		}

	}
	while(0 == ende);

	// cleaning the memory and quit the game
	for(int i = 0; i < num_player; i++)
	{
		SDL_KillThread(thread[i]);
		delete player[i];
	}

//	SDL_Delay(10);
	return 0;
}

void help()
{
	cout << "snake -L [Level (Dateiname)] -P [Anzahl Spieler]\n\n" << endl;
	cout << "Befehl:\t\tFunktion:\n\n";
	cout << "-L\t\tLevel (Dateiname)" << endl;
	cout << "-P\t\tSpieleranzahl" << endl;
	cout << "-h\t\tDiese Hilfe" << endl;
	cout << "-?\t\tSiehe -H" << endl;
}
